/* $Id: option.c,v 2.0 1998/03/25 05:43:54 svmcguir Exp $ */

#include <stdlib.h>
#include <math.h>
#include "path.h"
#include "option.h"
#include "util.h"

void optionDataAllocate(optionData *od, pathData *pd){
  /* Allocates od.payoff[0:time-1][0:path-1] using od.pData.times
   * and od.params.pData.paths.
   */

  allocate2dDouble(&(od->payoff),     pd->times, pd->meshPaths);
  allocate2dDouble(&(od->euroPrice),  pd->times, pd->meshPaths);
  allocate2dInt(   &(od->euroChoice), pd->times, pd->meshPaths);
}

void optionDataDeallocate(optionData *od, pathData *pd){
  /* Deallocates od.payoff[0:time-1][0:path-1] using od.pData.times
   * and od.params.pData.paths.
   */

  deallocate2dDouble(&(od->payoff),       pd->times, pd->meshPaths);
  deallocate2dDouble(&(od->euroPrice),    pd->times, pd->meshPaths);
  deallocate2dInt(   &(od->euroChoice),   pd->times, pd->meshPaths);
}

double optionPayoff(optionData *od, pathData *pd, double *price){
  /* Returns the payoff on the option described by od for the prices in
   * the array price.  The dimension of price is [0:od.underliers-1].
   * This is written for a max option.
   */

  double max,min;

  /* For a max (call) */

  max = dArrayMax(pd->underliers,price);
  return(max - od->strikePrice);


  /* For a min (put) */
  /*
    min = dArrayMin(pd->underliers,price); 
    return(od->strikePrice - min);
  */
}


void optionDataFill(optionData *od, pathData *pd){
  /* Fills od->payoff with payoffs for the underlier prices found in pd.
   * Also, deflates them with the risk free rate.
   */

  int time, path, choice;

  for(time=0; time < pd->times; time++){
    for(path=0; path < pd->meshPaths; path++){
      od->payoff[time][path] = exp(-time*od->riskFreeRate)*
        optionPayoff(od, pd, pd->price[time][path]);
    }
  }
}


